package com.ibm.xml.parser;

import java.io.Writer;
import java.util.Stack;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Node;

public class FormatPrintVisitor extends ToXMLStringVisitor implements Visitor {
   protected int currentIndent;
   protected int indent;
   protected boolean ispreserve;
   protected Stack preserves;
   protected boolean isprevtext;

   public FormatPrintVisitor(Writer var1, String var2, int var3) {
      super(var1, var2);
      this.indent = 2;
      this.ispreserve = false;
      this.preserves = new Stack();
      this.isprevtext = false;
      this.indent = var3;
   }

   public FormatPrintVisitor(Writer var1, String var2) {
      this(var1, var2, 2);
   }

   public FormatPrintVisitor(Writer var1) {
      this(var1, (String)null, 2);
   }

   public void visitDocumentPost(TXDocument var1) throws Exception {
      super.writer.write(10);
      super.visitDocumentPost(var1);
   }

   public void visitElementPre(TXElement var1) throws Exception {
      if (!this.ispreserve && !this.isprevtext) {
         Util.indent(super.writer, this.currentIndent);
      }

      super.visitElementPre(var1);
      this.currentIndent += this.indent;
      if (((Parent)var1).hasChildNodes()) {
         Child var2 = (Child)((Parent)var1).getFirstChild();
         this.preserves.push(new Boolean(this.ispreserve));
         if (var1.children.getLength() == 1 && var2.getNodeType() == 3) {
            this.ispreserve = true;
         } else {
            this.ispreserve = var1.isPreserveSpace();
         }
      }

      this.isprevtext = false;
   }

   public void visitElementPost(TXElement var1) throws Exception {
      this.currentIndent -= this.indent;
      if (((Parent)var1).hasChildNodes()) {
         if (!this.ispreserve && !this.isprevtext) {
            Util.indent(super.writer, this.currentIndent);
         }

         super.writer.write("</");
         super.writer.write(var1.getTagName());
         super.writer.write(">");
         this.ispreserve = (Boolean)this.preserves.pop();
      }

      this.isprevtext = false;
   }

   public void visitPIPre(TXPI var1) throws Exception {
      if (!this.ispreserve && !this.isprevtext) {
         Util.indent(super.writer, this.currentIndent);
      }

      super.visitPIPre(var1);
   }

   public void visitCommentPre(TXComment var1) throws Exception {
      if (!this.ispreserve && !this.isprevtext) {
         Util.indent(super.writer, this.currentIndent);
      }

      super.visitCommentPre(var1);
   }

   public void visitTextPre(TXText var1) throws Exception {
      if (!this.ispreserve && var1.getIsIgnorableWhitespace()) {
         this.isprevtext = false;
      } else if (var1 instanceof CDATASection) {
         if (!this.ispreserve && !this.isprevtext) {
            Util.indent(super.writer, this.currentIndent);
         }

         super.writer.write("<![CDATA[");
         super.writer.write(((TXCharacterData)var1).getData());
         super.writer.write("]]>");
      } else {
         super.writer.write(Util.backReference(((TXCharacterData)var1).getData(), super.encoding));
         this.isprevtext = true;
      }
   }

   public void visitDTDPre(DTD var1) throws Exception {
      Util.indent(super.writer, this.currentIndent);
      super.writer.write("<!DOCTYPE ");
      super.writer.write(var1.getName());
      super.writer.write(" ");
      if (var1.getExternalID() != null) {
         super.writer.write(var1.getExternalID().toString());
         super.writer.write(" ");
      }

      this.currentIndent += this.indent;
      if (var1.isPrintInternalDTD() && var1.getInternalSize() > 0) {
         super.writer.write("[\n");

         for(Node var2 = ((Parent)var1).getFirstChild(); var2 != null; var2 = var2.getNextSibling()) {
            if (var2.getNodeType() != 3) {
               try {
                  ((Child)var2).acceptPre(this);
               } catch (ToNextSiblingTraversalException var3) {
               }
            }
         }
      }

      throw new ToNextSiblingTraversalException();
   }

   public void visitDTDPost(DTD var1) throws Exception {
      this.currentIndent -= this.indent;
      if (var1.isPrintInternalDTD() && var1.getInternalSize() > 0) {
         Util.printSpace(super.writer, this.currentIndent);
         super.writer.write("]");
      }

      super.writer.write(">");
   }

   public void visitElementDeclPre(ElementDecl var1) throws Exception {
      Util.printSpace(super.writer, this.currentIndent);
      super.visitElementDeclPre(var1);
      super.writer.write(10);
   }

   public void visitAttlistPre(Attlist var1) throws Exception {
      Util.printSpace(super.writer, this.currentIndent);
      super.visitAttlistPre(var1);
      super.writer.write(10);
   }

   public void visitEntityDeclPre(EntityDecl var1) throws Exception {
      Util.printSpace(super.writer, this.currentIndent);

      try {
         super.visitEntityDeclPre(var1);
      } catch (ToNextSiblingTraversalException var2) {
      }

      super.writer.write(10);
      throw new ToNextSiblingTraversalException();
   }

   public void visitNotationPre(TXNotation var1) throws Exception {
      Util.printSpace(super.writer, this.currentIndent);
      super.visitNotationPre(var1);
      super.writer.write(10);
   }
}
