package com.ibm.xml.parser;

import org.w3c.dom.CharacterData;
import org.w3c.dom.DOMException;

public abstract class TXCharacterData extends Child implements CharacterData {
   static final long serialVersionUID = 8142858563528507880L;
   String data;

   public String getNodeValue() {
      return this.getData();
   }

   public String getData() {
      return this.data;
   }

   public void setNodeValue(String var1) {
      this.setData(var1);
   }

   public void setData(String var1) {
      this.data = var1;
      ((Child)this).clearDigest();
   }

   public int getLength() {
      return this.data.length();
   }

   public synchronized String substringData(int var1, int var2) throws DOMException {
      if (var1 >= 0 && var1 < this.data.length()) {
         if (var2 < 0) {
            throw new TXDOMException((short)1, "Invalid count: " + var2);
         } else {
            int var3 = var1 + var2;
            if (var3 > this.data.length()) {
               var3 = this.data.length();
            }

            return this.data.substring(var1, var3);
         }
      } else {
         throw new TXDOMException((short)1, "Out of bounds: " + var1 + ", the length of data is " + this.data.length());
      }
   }

   public synchronized void appendData(String var1) throws DOMException {
      StringBuffer var2 = new StringBuffer(this.data.length() + var1.length());
      var2.append(this.data);
      var2.append(var1);
      this.data = var2.toString();
      ((Child)this).clearDigest();
   }

   public synchronized void insertData(int var1, String var2) throws DOMException {
      if (var1 >= 0 && var1 <= this.data.length()) {
         StringBuffer var3 = new StringBuffer(this.data.length() + var2.length());
         var3.append(this.data);
         var3.insert(var1, var2);
         this.data = var3.toString();
         ((Child)this).clearDigest();
      } else {
         throw new TXDOMException((short)1, "offset is out of bounds: " + var1);
      }
   }

   public synchronized void deleteData(int var1, int var2) throws DOMException {
      if (var1 >= 0 && var1 < this.data.length()) {
         if (var2 < 0) {
            throw new TXDOMException((short)1, "Invalid count: " + var2);
         } else {
            int var3 = var1 + var2;
            if (var3 >= this.data.length()) {
               this.data = this.data.substring(0, var1);
            } else {
               this.data = this.data.substring(0, var1) + this.data.substring(var1 + var2);
            }

            ((Child)this).clearDigest();
         }
      } else {
         throw new TXDOMException((short)1, "Out of bounds: " + var1 + ", the length of data is " + this.data.length());
      }
   }

   public synchronized void replaceData(int var1, int var2, String var3) throws DOMException {
      if (var1 >= 0 && var1 < this.data.length()) {
         if (var2 < 0) {
            throw new TXDOMException((short)1, "Invalid count: " + var2);
         } else {
            int var4 = var1 + var2;
            if (var4 >= this.data.length()) {
               this.data = this.data.substring(0, var1) + var3;
            } else {
               this.data = this.data.substring(0, var1) + var3 + this.data.substring(var4);
            }

            ((Child)this).clearDigest();
         }
      } else {
         throw new TXDOMException((short)1, "Out of bounds: " + var1 + ", the length of data is " + this.data.length());
      }
   }
}
