package com.ibm.xml.parser;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

class TXNodeList implements NodeList, Serializable {
   static final long serialVersionUID = 3392852660870983354L;
   Vector nodes = new Vector();
   static EmptyNodeListImpl emptyNodeList = new EmptyNodeListImpl();

   public int getLength() {
      return this.nodes.size();
   }

   public Enumeration elements() {
      return this.nodes.elements();
   }

   public Node item(int var1) {
      return (Node)this.nodes.elementAt(var1);
   }

   public int indexOf(Node var1) {
      return this.nodes.indexOf(var1);
   }

   public void replace(int var1, Node var2) throws DOMException {
      if (var1 >= 0 && this.nodes.size() > var1) {
         Child var3 = (Child)this.item(var1);
         Child var4 = (Child)var3.getPreviousSibling();
         Child var5 = (Child)var3.getNextSibling();
         this.nodes.setElementAt(var2, var1);
         Child var6 = (Child)var2;
         var6.setPreviousSibling(var4);
         if (var4 != null) {
            var4.setNextSibling(var6);
         }

         var6.setNextSibling(var5);
         if (var5 != null) {
            var5.setPreviousSibling(var6);
         }

         var3.setPreviousSibling((Node)null);
         var3.setNextSibling((Node)null);
         var3.setParentNode((Node)null);
      } else {
         throw new TXDOMException((short)1, "com.ibm.xml.parser.TXNodeList#replace(): Wrong index: " + var1);
      }
   }

   public void insert(int var1, Node var2) throws DOMException {
      if (var1 >= 0 && var1 <= this.nodes.size()) {
         Child var3 = var1 > 0 ? (Child)this.nodes.elementAt(var1 - 1) : null;
         Child var4 = this.nodes.size() > var1 ? (Child)this.nodes.elementAt(var1) : null;
         this.nodes.insertElementAt(var2, var1);
         Child var5 = (Child)var2;
         var5.setPreviousSibling(var3);
         if (var3 != null) {
            var3.setNextSibling(var5);
         }

         var5.setNextSibling(var4);
         if (var4 != null) {
            var4.setPreviousSibling(var5);
         }

      } else {
         throw new TXDOMException((short)1, "com.ibm.xml.parser.TXNodeList#insert(): Wrong index: " + var1);
      }
   }

   public Node remove(int var1) throws DOMException {
      if (var1 >= 0 && this.nodes.size() > var1) {
         Child var2 = (Child)this.nodes.elementAt(var1);
         Child var3 = (Child)var2.getPreviousSibling();
         Child var4 = (Child)var2.getNextSibling();
         this.nodes.removeElementAt(var1);
         var2.setParentNode((Node)null);
         var2.setPreviousSibling((Node)null);
         var2.setNextSibling((Node)null);
         if (var3 != null) {
            var3.setNextSibling(var4);
         }

         if (var4 != null) {
            var4.setPreviousSibling(var3);
         }

         return var2;
      } else {
         throw new TXDOMException((short)1, "com.ibm.xml.parser.TXNodeList#remove(): Wrong index: " + var1);
      }
   }

   public void append(Node var1) {
      this.insert(this.nodes.size(), var1);
   }

   public void ensureCapacity(int var1) {
      this.nodes.ensureCapacity(var1);
   }

   public boolean equals(Object var1) {
      return this.equals(var1, true);
   }

   public synchronized boolean equals(Object var1, boolean var2) {
      if (var1 == null) {
         return false;
      } else if (!(var1 instanceof NodeList)) {
         return false;
      } else {
         NodeList var3 = (NodeList)var1;
         if (var3.getLength() != this.getLength()) {
            return false;
         } else {
            for(int var4 = 0; var4 < var3.getLength(); ++var4) {
               Node var5 = var3.item(var4);
               if (!((Child)var5).equals(this.item(var4), var2)) {
                  return false;
               }
            }

            return true;
         }
      }
   }

   public int hashCode() {
      return 0;
   }
}
