package com.ibm.xml.parser.util;

import com.ibm.xml.parser.AttDef;
import com.ibm.xml.parser.Attlist;
import com.ibm.xml.parser.Child;
import com.ibm.xml.parser.ContentModel;
import com.ibm.xml.parser.DTD;
import com.ibm.xml.parser.ElementDecl;
import com.ibm.xml.parser.EntityDecl;
import com.ibm.xml.parser.ExternalID;
import com.ibm.xml.parser.GeneralReference;
import com.ibm.xml.parser.Parent;
import com.ibm.xml.parser.PseudoNode;
import com.ibm.xml.parser.StylesheetPI;
import com.ibm.xml.parser.TXAttribute;
import com.ibm.xml.parser.TXCharacterData;
import com.ibm.xml.parser.TXDocument;
import com.ibm.xml.parser.TXElement;
import com.ibm.xml.parser.TXNotation;
import com.ibm.xml.parser.TXText;
import java.util.Enumeration;
import javax.swing.tree.TreeNode;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.Node;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;

public class TreeFactory extends TXDocument implements TreeNode {
   public String convertValueToText(Object var1) {
      String var2 = "";
      switch (((Child)var1).getNodeType()) {
         case 1:
            var2 = this.toStringElement((TXElement)var1, (String)null);
            break;
         case 2:
            var2 = this.toStringAttribute((TXAttribute)var1, (String)null);
            break;
         case 3:
            var2 = this.toStringText((Text)var1, (String)null);
            break;
         case 4:
            var2 = this.toStringCDATA((CDATASection)var1, (String)null);
            break;
         case 5:
            var2 = this.toStringGeneralReference((GeneralReference)var1, (String)null);
            break;
         case 6:
            var2 = this.toStringEntityDecl((EntityDecl)var1, (String)null);
            break;
         case 7:
            var2 = this.toStringPI((ProcessingInstruction)var1, (String)null);
            break;
         case 8:
            var2 = this.toStringComment((Comment)var1, (String)null);
            break;
         case 9:
            var2 = this.toStringDocument((TXDocument)var1, (String)null);
            break;
         case 10:
            var2 = this.toStringDTD((DTD)var1, (String)null);
         case 11:
         case 13:
         case 14:
         case 15:
         case 16:
         case 17:
         case 18:
         case 19:
         default:
            break;
         case 12:
            var2 = this.toStringNotation((TXNotation)var1, (String)null);
            break;
         case 20:
            var2 = this.toStringElementDecl((ElementDecl)var1, (String)null);
            break;
         case 21:
            var2 = this.toStringAttlist((Attlist)var1, (String)null);
            break;
         case 22:
            var2 = this.toStringAttDef((AttDef)var1, (String)null);
            break;
         case 23:
            var2 = this.toStringPseudoNode((PseudoNode)var1, (String)null);
      }

      return var2;
   }

   public String toStringAttDef(AttDef var1, String var2) {
      return var1.getName();
   }

   public String toStringAttlist(Attlist var1, String var2) {
      return "!ATTLIST " + var1.getName();
   }

   public String toStringDTD(DTD var1, String var2) {
      return "!DOCTYPE " + var1.getName();
   }

   public String toStringElementDecl(ElementDecl var1, String var2) {
      return "!ELEMENT " + var1.getName();
   }

   public String toStringEntityDecl(EntityDecl var1, String var2) {
      return "!ENTITY " + (var1.isParameter() ? "% " : "") + var1.getNodeName();
   }

   public String toStringComment(Comment var1, String var2) {
      return "<!--" + var1.getData() + "-->";
   }

   public String toStringDocument(TXDocument var1, String var2) {
      return "[DOCUMENT]";
   }

   public String toStringElement(TXElement var1, String var2) {
      StringBuffer var3 = new StringBuffer();
      var3.append("<");
      var3.append(var1.getNodeName());
      Enumeration var4 = var1.attributeElements();

      while(var4.hasMoreElements()) {
         var3.append(" ");
         var3.append(((TXAttribute)var4.nextElement()).toXMLString(var2));
      }

      var3.append(">");
      return var3.toString();
   }

   public String toStringNotation(TXNotation var1, String var2) {
      return "!NOTATION " + var1.getNodeName();
   }

   public String toStringPI(ProcessingInstruction var1, String var2) {
      return "<?" + var1.getTarget() + var1.getData() + "?>";
   }

   public String toStringText(Text var1, String var2) {
      return "\"" + TXText.makePrintable(var1.getData()) + "\"";
   }

   public String toStringCDATA(CDATASection var1, String var2) {
      return "<![CDATA[" + var1.getData() + "]]>";
   }

   public String toStringAttribute(TXAttribute var1, String var2) {
      return "";
   }

   public String toStringGeneralReference(GeneralReference var1, String var2) {
      return "&" + var1.getName() + ";";
   }

   public String toStringPseudoNode(PseudoNode var1, String var2) {
      return ((TXCharacterData)var1).getData();
   }

   public Element createElement(String var1) {
      TreeElement var2 = new TreeElement(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public TXText createTextNode(String var1, boolean var2) {
      TreeText var3 = new TreeText(var1);
      ((Child)var3).setFactory(this);
      ((TXText)var3).setIsIgnorableWhitespace(var2);
      return var3;
   }

   public TXText createTextNode(char[] var1, int var2, int var3, boolean var4) {
      TreeText var5 = new TreeText(new String(var1, var2, var3));
      ((Child)var5).setFactory(this);
      ((TXText)var5).setIsIgnorableWhitespace(var4);
      return var5;
   }

   public CDATASection createCDATASection(String var1) {
      TreeCDATASection var2 = new TreeCDATASection(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public Comment createComment(String var1) {
      TreeComment var2 = new TreeComment(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public ProcessingInstruction createProcessingInstruction(String var1, String var2) {
      TreePI var3 = new TreePI(var1, var2);
      ((Child)var3).setFactory(this);
      return var3;
   }

   public StylesheetPI createStylesheetPI(String var1, String var2, String var3, String var4, String var5) {
      TreeStylesheetPI var6 = new TreeStylesheetPI(var1, var2, var3, var4, var5);
      ((Child)var6).setFactory(this);
      return var6;
   }

   public DTD createDTD() {
      TreeDTD var1 = new TreeDTD();
      ((Child)var1).setFactory(this);
      return var1;
   }

   public DTD createDTD(String var1, ExternalID var2) {
      TreeDTD var3 = new TreeDTD(var1, var2);
      ((Child)var3).setFactory(this);
      return var3;
   }

   public ElementDecl createElementDecl(String var1, ContentModel var2) {
      TreeElementDecl var3 = new TreeElementDecl(var1, var2);
      ((Child)var3).setFactory(this);
      return var3;
   }

   public Attlist createAttlist(String var1) {
      TreeAttlist var2 = new TreeAttlist(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public AttDef createAttDef(String var1) {
      TreeAttDef var2 = new TreeAttDef(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public EntityDecl createEntityDecl(String var1, String var2, boolean var3) {
      TreeEntity var4 = new TreeEntity(var1, var2, var3);
      ((Child)var4).setFactory(this);
      return var4;
   }

   public EntityDecl createEntity(String var1, ExternalID var2, boolean var3, String var4) {
      TreeEntity var5 = new TreeEntity(var1, var2, var3, var4);
      ((Child)var5).setFactory(this);
      return var5;
   }

   public TXNotation createNotation(String var1, ExternalID var2) {
      TreeNotation var3 = new TreeNotation(var1, var2);
      ((Child)var3).setFactory(this);
      return var3;
   }

   public EntityReference createEntityReference(String var1) {
      TreeGeneralReference var2 = new TreeGeneralReference(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public PseudoNode createPseudoNode(String var1) {
      TreePseudoNode var2 = new TreePseudoNode(var1);
      ((Child)var2).setFactory(this);
      return var2;
   }

   public Enumeration children() {
      return ((Parent)this).elements();
   }

   public boolean getAllowsChildren() {
      return true;
   }

   public TreeNode getChildAt(int var1) {
      return (TreeNode)((Parent)this).getChildNodes().item(var1);
   }

   public int getChildCount() {
      return ((Parent)this).getChildNodes().getLength();
   }

   public int getIndex(TreeNode var1) {
      int var2 = 0;

      for(Node var3 = ((Parent)this).getFirstChild(); var3 != null; var3 = var3.getNextSibling()) {
         if (var3 == var1) {
            return var2;
         }

         ++var2;
      }

      return -1;
   }

   public TreeNode getParent() {
      return (TreeNode)((Child)this).getParentNode();
   }

   public boolean isLeaf() {
      return false;
   }

   public String toString() {
      return ((TreeFactory)((TXDocument)this).getFactory()).convertValueToText(this);
   }
}
